# -------------------------------------------------------------------------------
# Copyright IBM Corp. 2017
# 
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
# http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------
from six import with_metaclass
import math
import matplotlib as mpl
from matplotlib import cm

'''
Static class that provides consistent color palette for all charts
'''
class Colors(with_metaclass( 
        type("",(type,),{
            "colormap":cm.jet,
            "__getitem__":lambda cls, key: cls.colormap(key),
            "__getattr__":lambda cls, key: cls.colormap(key)
        }), object
    )):

    @staticmethod
    def hexRGB(X):
        tupleColor = Colors.colormap(X)
        return str(mpl.colors.rgb2hex( tupleColor ).upper())

    #TODO: provide user configured colormap palette
    pass
