<div>
    <h2>No Google Maps API Key</h2>
    <p>
        Some functionality of the Google maps renderer requires an API key.
    </p>
    <p>
        You can obtain an API key from the <a href="https://code.google.com/apis/console/" target="_blank">Google API Console</a>.
        After <a href="https://support.google.com/googleapi/answer/6158862?hl=en&ref_topic=7013279" target="_blank">getting the API key</a>,
        it must be enabled for the <a href="https://console.developers.google.com/projectselector/apis/library" target="_blank">Google Maps Javascript API and the Geocoding API</a>.
        Once enabled, you may enter the API key in the Options dialog.
    </p>
</div>