{%extends resModule + ":baseChartOptionsDialogBody.html"%}
{%block staticFigure%}
{%endblock%}
{%block customFormFields%}
<div class="form-group col-sm-8" style="padding: 0px 0px 0px 5px;">
	<label for="title{{prefix}}"><a href="https://support.google.com/googleapi/answer/6158862?hl=en&ref_topic=7013279" target="_blank">Google Maps API Key</a>: <i class="fa fa-question-circle" title="Get a Google Maps API key by accessing the Google API Console"></i></label>
	<input type="text" class="form-control" id="googlemapapikey{{prefix}}" name="googlemapapikey" value="{{this.options.get("googlemapapikey") or ""}}">
</div>
{%endblock%}